from django.db import models
from wagtail.admin.edit_handlers import (
    FieldPanel,
    ObjectList,
    PageChooserPanel,
    StreamFieldPanel,
    TabbedInterface,
)
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.core import blocks
from wagtail.core.fields import RichTextField, StreamField
from wagtail.core.models import Page
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtail.search import index


class SenatorHomePage(Page):
    name = models.CharField("jméno senátora", max_length=250)
    profession = models.CharField("profese", max_length=250)
    district = models.CharField("info o okrsku", max_length=250)
    image = models.ForeignKey(
        "wagtailimages.Image", on_delete=models.PROTECT, null=True
    )
    about = models.TextField("o mně")
    my_work = models.TextField("má práce v senátu")

    content_panels = Page.content_panels + [
        FieldPanel("name"),
        FieldPanel("profession"),
        FieldPanel("district"),
        ImageChooserPanel("image"),
        FieldPanel("about"),
        FieldPanel("my_work"),
    ]

    subpage_types = [
        "senator.SenatorNewsIndexPage",
        "senator.SenatorTextPage",
        "senator.SenatorMultiTextPage",
        "senator.SenatorAboutPage",
    ]

    class Meta:
        verbose_name = "[senátor] Úvodní stránka"

    def get_context(self, request):
        context = super().get_context(request)

        articles = (
            self.get_descendants()
            .type(SenatorNewsPage)
            .live()
            .order_by("-senatornewspage__date")[:3]
        )
        context["articles"] = articles

        try:
            context["news_url"] = (
                self.get_children()
                .type(SenatorNewsIndexPage)
                .live()
                .get()
                .get_url(request)
            )
        except Page.DoesNotExist:
            context["news_url"] = "#"

        return context


class SenatorNewsIndexPage(Page):
    parent_page_types = ["senator.SenatorHomePage"]
    subpage_types = ["senator.SenatorNewsPage"]

    class Meta:
        verbose_name = "[senátor] Přehled novinek"

    def get_context(self, request):
        context = super().get_context(request)
        articles = self.get_children().live().order_by("-senatornewspage__date")
        context["articles"] = articles
        return context


class SenatorNewsPage(Page):
    date = models.DateField("datum")
    perex = models.TextField("perex")
    body = RichTextField("článek", blank=True)
    image = models.ForeignKey(
        "wagtailimages.Image", on_delete=models.SET_NULL, null=True, blank=True
    )

    search_fields = Page.search_fields + [
        index.SearchField("perex"),
        index.SearchField("body"),
        index.FilterField("date"),
    ]

    content_panels = Page.content_panels + [
        FieldPanel("date"),
        FieldPanel("perex"),
        FieldPanel("body", classname="full"),
        ImageChooserPanel("image"),
    ]

    parent_page_types = ["senator.SenatorNewsIndexPage"]
    subpage_types = []

    class Meta:
        verbose_name = "[senátor] Novinka"


class SenatorTextPage(Page):
    body = RichTextField("obsah", blank=True)

    search_fields = Page.search_fields + [
        index.SearchField("body"),
    ]

    content_panels = Page.content_panels + [
        FieldPanel("body", classname="full"),
    ]

    class Meta:
        verbose_name = "[senátor] Textová stránka"


class SenatorMultiTextPage(Page):
    body = StreamField([("paragraph", blocks.RichTextBlock())])

    search_fields = Page.search_fields + [
        index.SearchField("body"),
    ]

    content_panels = Page.content_panels + [
        StreamFieldPanel("body"),
    ]

    class Meta:
        verbose_name = "[senátor] Flexibilní textová stránka"


class SenatorAboutPage(Page):
    profile = RichTextField("profil", blank=True)
    image = models.ForeignKey("wagtailimages.Image", on_delete=models.PROTECT)
    body = StreamField([("paragraph", blocks.RichTextBlock())])

    search_fields = Page.search_fields + [
        index.SearchField("profile"),
        index.SearchField("body"),
    ]

    content_panels = Page.content_panels + [
        FieldPanel("profile", classname="full"),
        ImageChooserPanel("image"),
        StreamFieldPanel("body"),
    ]

    class Meta:
        verbose_name = "[senátor] O mně"


@register_setting
class SenatorWebSettings(BaseSetting):
    title_first_name = models.CharField("jméno do titulku", max_length=250)
    title_last_name = models.CharField("příjmení do titulku", max_length=250)

    email = models.EmailField("kontaktní email", blank=True, null=True)
    facebook = models.URLField("Facebook URL", blank=True, null=True)
    instagram = models.URLField("Instagram URL", blank=True, null=True)
    twitter = models.URLField("Twitter URL", blank=True, null=True)
    linkedin = models.URLField("LinkedIn URL", blank=True, null=True)

    client = models.CharField("zadavatel", max_length=250)
    created_by = models.CharField("zhotovitel", max_length=250)
    cookies_page = models.ForeignKey(
        "wagtailcore.Page", null=True, on_delete=models.SET_NULL, related_name="+"
    )

    first_tab_panels = [
        FieldPanel("title_first_name"),
        FieldPanel("title_last_name"),
        FieldPanel("client"),
        FieldPanel("created_by"),
        PageChooserPanel("cookies_page"),
    ]

    second_tab_panels = [
        FieldPanel("email"),
        FieldPanel("facebook"),
        FieldPanel("instagram"),
        FieldPanel("twitter"),
        FieldPanel("linkedin"),
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(first_tab_panels, heading="Nastavení"),
            ObjectList(second_tab_panels, heading="Sociální sítě"),
        ]
    )

    class Meta:
        verbose_name = "[senátor] Nastavení webu"
